package ms.warpzone.warppay.manager;

import android.util.Log;
import android.widget.Toast;

import java.util.ArrayList;

import ms.warpzone.warppay.data.models.local.User;
import ms.warpzone.warppay.data.models.rest.RestProduct;
import ms.warpzone.warppay.data.models.rest.RestTransaction;
import ms.warpzone.warppay.data.models.rest.RestUser;
import ms.warpzone.warppay.orderList.Order;
import retrofit.Callback;
import retrofit.Response;
import retrofit.Retrofit;

public class DataManager {
    private User currentUser;

    private Boolean is_guest=false;
    private double totalAmount;
    private ArrayList<Order> orderList;
    private static DataManager ourInstance = new DataManager();
    private String lastCardId;

    public static DataManager getInstance() {
        return ourInstance;
    }

    private DataManager() {
        this.orderList = new ArrayList<Order>();
    }

    public void saveCurrentUser() {

        RestUser rest_user = RestUser.fromLocalUser(this.currentUser);
        MainManager.getInstance().getRestService().saveUser(rest_user.getUserid(), rest_user).enqueue(new Callback<RestUser>() {
            @Override
            public void onResponse(Response<RestUser> response, Retrofit retrofit) {
                if(response.code() == 403) {
                    Toast.makeText(MainManager.getInstance().getMainActivity(), "Eine Karte ist für den User schon vorhanden", Toast.LENGTH_LONG).show();
                } else {
                    MainManager.getInstance().getCurrentUser().save();
                }
            }

            @Override
            public void onFailure(Throwable t) {
                Log.d("REST", t.getMessage());
            }
        });

    }

    public Boolean getIs_guest() {
        return is_guest;
    }

    public void setIs_guest(Boolean is_guest) {
        this.is_guest = is_guest;
    }

    public void performPayment(Boolean cash) {
        for (Order order:this.orderList) {
            RestTransaction transaction = new RestTransaction();
            transaction.setProduct(RestProduct.fromLocalProduct(order.getProduct()));
            transaction.setTrans_type(2);
            if(this.is_guest || cash)
                transaction.setCash_paid(true);
            MainManager.getInstance().getRestService().saveTransaction(this.currentUser.getUserid(), transaction).enqueue(new Callback<Void>() {
                @Override
                public void onResponse(Response<Void> response, Retrofit retrofit) {

                }

                @Override
                public void onFailure(Throwable t) {
                    Log.d("REST", t.getMessage());
                }
            });
        }
        if(!this.is_guest && !cash) {
            this.currentUser.setCredit(this.currentUser.getCredit() - this.totalAmount);
            this.currentUser.save();
        }
        this.totalAmount = 0.0;
    }

    public void clearCurrentUser() {
        this.currentUser = null;
        this.totalAmount = 0.0;
        this.is_guest=false;
        this.orderList = new ArrayList<Order>();
    }

    public User getCurrentUser() {
        return currentUser;
    }

    public void setCurrentUser(User currentUser) {
        this.currentUser = currentUser;
    }

    public double getTotalAmount() {
        return totalAmount;
    }

    public void addOrder(Order order) {
        this.orderList.add(order);
        this.totalAmount += order.getProduct().getPrice();
    }
    public void removeOrder(Order order) {
        this.orderList.remove(order);
        this.totalAmount -= order.getProduct().getPrice();
    }
    public void setTotalAmount(double totalAmount) {
        this.totalAmount = totalAmount;
    }

    public void setLastCardId(String lastCardId) {
        this.lastCardId = lastCardId;
    }

    public String getLastCardId() {
        return lastCardId;
    }
}
